package sun.management;

import javax.management.ObjectName;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import com.sun.management.GarbageCollectionNotificationInfo;
import javax.management.Notification;
import javax.management.MBeanNotificationInfo;
import com.sun.management.GcInfo;
import java.util.Iterator;
import java.util.List;
import java.lang.management.MemoryPoolMXBean;
import java.lang.management.ManagementFactory;
import com.sun.management.GarbageCollectorMXBean;

class GarbageCollectorImpl extends MemoryManagerImpl implements GarbageCollectorMXBean
{
    private String[] poolNames;
    private GcInfoBuilder gcInfoBuilder;
    private static final String notifName = "javax.management.Notification";
    private static final String[] gcNotifTypes;
    private static long seqNumber;
    
    GarbageCollectorImpl(String s) {
        super(s);
        this.poolNames = null;
    }
    
    @Override
    public native long getCollectionCount();
    
    @Override
    public native long getCollectionTime();
    
    synchronized String[] getAllPoolNames() {
        if (this.poolNames == null) {
            List<MemoryPoolMXBean> memoryPoolMXBeans = ManagementFactory.getMemoryPoolMXBeans();
            this.poolNames = new String[memoryPoolMXBeans.size()];
            int n = 0;
            Iterator<Object> iterator = memoryPoolMXBeans.iterator();
            while (iterator.hasNext()) {
                this.poolNames[n++] = ((MemoryPoolMXBean)iterator.next()).getName();
            }
        }
        return this.poolNames;
    }
    
    private synchronized GcInfoBuilder getGcInfoBuilder() {
        if (this.gcInfoBuilder == null) {
            this.gcInfoBuilder = new GcInfoBuilder(this, this.getAllPoolNames());
        }
        return this.gcInfoBuilder;
    }
    
    @Override
    public GcInfo getLastGcInfo() {
        return this.getGcInfoBuilder().getLastGcInfo();
    }
    
    @Override
    public MBeanNotificationInfo[] getNotificationInfo() {
        return new MBeanNotificationInfo[] { new MBeanNotificationInfo(GarbageCollectorImpl.gcNotifTypes, "javax.management.Notification", "GC Notification") };
    }
    
    private static long getNextSeqNumber() {
        return ++GarbageCollectorImpl.seqNumber;
    }
    
    void createGCNotification(long timeStamp, String s, String gcAction, String gcCause, GcInfo gcInfo) {
        if (!this.hasListeners()) {
            return;
        }
        Notification notification = new Notification("com.sun.management.gc.notification", this.getObjectName(), getNextSeqNumber(), timeStamp, s);
        notification.setUserData(GarbageCollectionNotifInfoCompositeData.toCompositeData(new GarbageCollectionNotificationInfo(s, gcAction, gcCause, gcInfo)));
        this.sendNotification(notification);
        return;
    }
    
    @Override
    public synchronized void addNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) {
        int hasListeners = this.hasListeners();
        super.addNotificationListener(notificationListener, notificationFilter, o);
        int hasListeners2 = this.hasListeners();
        if (hasListeners == 0 && hasListeners2) {
            this.setNotificationEnabled(this, true);
        }
    }
    
    @Override
    public synchronized void removeNotificationListener(NotificationListener notificationListener) throws ListenerNotFoundException {
        int hasListeners = this.hasListeners();
        super.removeNotificationListener(notificationListener);
        int hasListeners2 = this.hasListeners();
        if (hasListeners && hasListeners2 == 0) {
            this.setNotificationEnabled(this, false);
        }
    }
    
    @Override
    public synchronized void removeNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws ListenerNotFoundException {
        int hasListeners = this.hasListeners();
        super.removeNotificationListener(notificationListener, notificationFilter, o);
        int hasListeners2 = this.hasListeners();
        if (hasListeners && hasListeners2 == 0) {
            this.setNotificationEnabled(this, false);
        }
    }
    
    @Override
    public ObjectName getObjectName() {
        return Util.newObjectName("java.lang:type=GarbageCollector", this.getName());
    }
    
    native void setNotificationEnabled(GarbageCollectorMXBean p0, boolean p1);
    
    static {
        gcNotifTypes = new String[] { "com.sun.management.gc.notification" };
        GarbageCollectorImpl.seqNumber = 0L;
    }
}
